package net.sf.doolin.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.doolin.util.IdentityConverter;
import net.sf.doolin.util.ItemConverter;
import net.sf.doolin.util.PropertyFunction;
import net.sf.doolin.util.PropertyItemConverter;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.Maps;

/**
 * This class provides utility methods for collection conversions.
 * 
 * @author Damien Coraboeuf
 * 
 */
@SuppressWarnings("deprecation")
public class ListUtils {

	/**
	 * Converts values from a source map to a target map using a
	 * {@link ItemConverter converter} for the values.
	 * 
	 * @param <T>
	 *            Type of the value in the source map
	 * @param <K>
	 *            Type of key for both maps
	 * @param <V>
	 *            Type of value for the target map
	 * @param map
	 *            Source map
	 * @param valueExtractor
	 *            Converter for the value
	 * @return Target map
	 * @deprecated Use
	 *             {@link Maps#transformValues(Map, com.google.common.base.Function)}
	 *             instead.
	 */
	@Deprecated
	public static <T, K, V> Map<K, V> convertMap(Map<K, ? extends T> map,
			ItemConverter<T, V> valueExtractor) {
		Map<K, V> result = new HashMap<K, V>();
		for (Map.Entry<K, ? extends T> entry : map.entrySet()) {
			K key = entry.getKey();
			T t = entry.getValue();
			V v = valueExtractor.convert(t);
			result.put(key, v);
		}
		return result;
	}

	/**
	 * Converts values from a source map to a target map, by getting a property
	 * from the source value.
	 * 
	 * @param <T>
	 *            Type of the value in the source map
	 * @param <K>
	 *            Type of key for both maps
	 * @param <V>
	 *            Type of value for the target map
	 * @param map
	 *            Source map
	 * @param property
	 *            Property to take from the source value
	 * @return Target map
	 * @see PropertyItemConverter
	 */
	public static <T, K, V> Map<K, V> convertMap(Map<K, ? extends T> map,
			String property) {
		return Maps.transformValues(map, new PropertyFunction<T, V>(property));
	}

	/**
	 * Extracts a sub-list by evaluating a {@link Predicate} on the source list
	 * 
	 * @param <T>
	 *            Type of object in the list
	 * @param list
	 *            Source list
	 * @param predicate
	 *            Predicate to evaluate
	 * @return List of objects in the source list the predicate has returned
	 *         <code>true</code> for
	 */
	public static <T> List<T> extractList(Collection<? extends T> list,
			Predicate<T> predicate) {
		List<T> result = new ArrayList<T>();
		for (T t : list) {
			if (predicate.apply(t)) {
				result.add(t);
			}
		}
		return result;
	}

	/**
	 * Finds the first element in a list which is evaluated to <code>true</code>
	 * by a {@link Predicate}.
	 * 
	 * @param <E>
	 *            Type of item in the list
	 * @param list
	 *            List
	 * @param predicate
	 *            Predicate to apply
	 * @return First found item or <code>null</code> if none is found
	 */
	public static <E> E find(List<? extends E> list, Predicate<E> predicate) {
		int index = 0;
		for (E e : list) {
			if (predicate.apply(e)) {
				return e;
			}
			index++;
		}
		return null;
	}

	/**
	 * Converts a list to a map by extracting keys and values from each element
	 * of the list
	 * 
	 * @param <T>
	 *            Type of element in the list
	 * @param <K>
	 *            Type for the key
	 * @param <V>
	 *            Type for the value
	 * @param values
	 *            Source collection
	 * @param keyExtractor
	 *            Converter for the key
	 * @param valueExtractor
	 *            Converter for the value
	 * @return Map
	 */
	public static <T, K, V> Map<K, V> indexMap(Collection<? extends T> values,
			Function<T, K> keyExtractor, Function<T, V> valueExtractor) {
		Map<K, V> map = new HashMap<K, V>();
		for (T t : values) {
			K k = keyExtractor.apply(t);
			V v = valueExtractor.apply(t);
			map.put(k, v);
		}
		return map;
	}

	/**
	 * Converts a list to a map by extracting keys and values from each element
	 * of the list
	 * 
	 * @param <T>
	 *            Type of element in the list
	 * @param <K>
	 *            Type for the key
	 * @param <V>
	 *            Type for the value
	 * @param values
	 *            Source collection
	 * @param keyExtractor
	 *            Converter for the key
	 * @param valueExtractor
	 *            Converter for the value
	 * @return Map
	 * @deprecated Use {@link #indexMap(Collection, Function, Function)} instead
	 */
	@Deprecated
	public static <T, K, V> Map<K, V> indexMap(Collection<? extends T> values,
			ItemConverter<T, K> keyExtractor, ItemConverter<T, V> valueExtractor) {
		Map<K, V> map = new HashMap<K, V>();
		for (T t : values) {
			K k = keyExtractor.convert(t);
			V v = valueExtractor.convert(t);
			map.put(k, v);
		}
		return map;
	}

	/**
	 * Converts a list to a map by extracting keys and values from each element
	 * of the list using a property
	 * 
	 * @param <T>
	 *            Type of element in the list
	 * @param <K>
	 *            Type for the key
	 * @param <V>
	 *            Type for the value
	 * @param list
	 *            Source collection
	 * @param keyProperty
	 *            Property for the key
	 * @param valueProperty
	 *            Property for the value
	 * @return Map
	 */
	public static <T, K, V> Map<K, V> indexMap(Collection<? extends T> list,
			String keyProperty, String valueProperty) {
		return indexMap(list, new PropertyFunction<T, K>(keyProperty),
				new PropertyFunction<T, V>(valueProperty));
	}

	/**
	 * Converts a list to a map by extracting keys from each element of the
	 * list. The values for the map are the elements of the list.
	 * 
	 * @param <K>
	 *            Type for the key
	 * @param <V>
	 *            Type for the list and the map value
	 * @param values
	 *            Source collection
	 * @param keyExtractor
	 *            Converter for the key
	 * @return Map
	 * @see IdentityConverter
	 * @deprecated Use {@link Maps#uniqueIndex(Iterable, Function)} instead
	 */
	@Deprecated
	public static <K, V> Map<K, V> indexMap(List<? extends V> values,
			ItemConverter<V, K> keyExtractor) {
		return indexMap(values, keyExtractor, new IdentityConverter<V>());
	}

	/**
	 * Converts a list to a map by extracting keys from each element of the list
	 * using a property name. The values for the map are the elements of the
	 * list.
	 * 
	 * @param <K>
	 *            Type for the key
	 * @param <V>
	 *            Type for the list and the map value
	 * @param values
	 *            Source collection
	 * @param propertyName
	 *            Property for the key
	 * @return Map
	 */
	public static <K, V> Map<K, V> indexMap(List<V> values,
			final String propertyName) {
		return Maps.uniqueIndex(values,
				new PropertyFunction<V, K>(propertyName));
	}

	/**
	 * Returns the index of the first element of the list that is evaluated to
	 * <code>true</code> by the predicate.
	 * 
	 * @param <E>
	 *            Type of object in the list
	 * @param list
	 *            List
	 * @param predicate
	 *            Predicate to apply
	 * @return Index of the first positive element or -1 if not found
	 */
	public static <E> int indexOf(List<? extends E> list, Predicate<E> predicate) {
		int index = 0;
		for (E e : list) {
			if (predicate.apply(e)) {
				return index;
			}
			index++;
		}
		return -1;
	}

	/**
	 * Creates a sub-list that contains items referenced by a list of indexes.
	 * 
	 * @param <T>
	 *            Type of object in the list
	 * @param list
	 *            Source list
	 * @param indexes
	 *            List of indexes in the source list
	 * @return Sub-list
	 */
	public static <T> List<T> sublist(List<? extends T> list, int[] indexes) {
		if (indexes == null || indexes.length == 0) {
			return Collections.emptyList();
		} else {
			List<T> result = new ArrayList<T>(indexes.length);
			for (int index : indexes) {
				T item = list.get(index);
				result.add(item);
			}
			return result;
		}
	}

	private ListUtils() {
	}
}
